﻿;<=====  System Settings  ====================================================>
#SingleInstance, Force
#NoEnv
SetBatchLines, -1
OnExit, Exit

;<=====  Settings  ===========================================================>
logFile := "FailureLog.txt"   ; Log file for unreachable hosts
hostList := "HostList.txt"    ; List of host names to check
CheckInterval := 10           ; Check interval in seconds
Rows := 0                     ; Rows of boxes to display. 0 is AutoSize
AutoTracert := 0              ; If true, tracert is started on failed pings automatically

;<=====  Read in hostList  ===================================================>
FileRead, hostfile, %hostList%
hosts := Object()
Loop, Parse, hostfile, `n, `r
   hosts.Insert(A_LoopField)

;<=====  Startup  ============================================================>
DllCall("AllocConsole")
WinHide % "ahk_id " DllCall("GetConsoleWindow", "ptr")

if !Rows {
   SysGet, Mon, MonitorWorkArea
   Rows := Floor(MonBottom / 45) - 2
}

;<=====  Setup GUI  ==========================================================>
Gui +ToolWindow 
;Gui +AlwaysOnTop +ToolWindow
Gui, Font, Arial
RowX := 5
RowY := 5
Loop % hosts.MaxIndex()
{
   if (A_Index != 1) && !Mod(A_Index - 1, Rows){
      RowX += 155
      RowY := 5
   }
   host := hosts[A_Index]
   StringSplit, host, host, .
   Gui, Add, Picture, x%RowX% y%RowY% w150 h40 0x4000000 vi%A_Index%, %A_ScriptDir%\Images\Yellow.jpg
   Gui, Add, Text, xp+5 yp+5 w100 +Center, %host1%
   Gui, Add, Button, xp+100 yp w40 h13 vh%A_Index% gRDP, rdp
   Gui, Add, Text, xp-100 yp+15 w100 +Center vp%A_Index%, n/a
   Gui, Add, Button, xp+100 yp w40 h13 vb%A_Index% gStartTrace, trcrt
   RowY += 45
}
Gui, Add, Button, x5 w150 gOpenLog Default, Open Log
Gui, Add, Button, x5 yp+25 w150 gClearLog, Clear Log
Gui, Add, Button, x5 yp+25 w150 gFlushDNS, Clear DNS Cache
Width := (Ceil(Rowx/155) * 160)
XPos := (A_ScreenWidth - Width)-10
Gui, Show, x10 y10 w%Width% ;, x%XPos% yCenter w%Width% 
;WinMove, A,, %XPos%, %Y%, 

;<=====  Update Timer  =======================================================>
delay := CheckInterval * 1000
GoSub, CheckHosts
SetTimer, CheckHosts, %delay%
return

;<=====  Subs  ===============================================================>
CheckHosts:
   Loop, % hosts.MaxIndex()
   {
      cntrl = p%A_Index%
      hcntrl = h%A_Index%
      icntrl = i%A_Index%
      GuiControl,, %bcntrl%, %A_ScriptDir%\Images\Yellow.jpg
      ; Do the thing
      ping := CheckServer(hosts[A_Index])
      if ((ping == -1)||ping == ""){
         ; Update gui and Log unreachable host
         GuiControl,, %icntrl%, %A_ScriptDir%\Images\Red.jpg
         GuiControl,, %cntrl%, NO REPLY
         LogE(hosts[A_Index], logFile)
         if AutoTracert
            StartTrace(hosts[A_Index], logFile)
      } else {
         ; Update gui with ping
         GuiControl,, %icntrl%, %A_ScriptDir%\Images\Green.jpg
         GuiControl,, %cntrl%, %ping%
      }
   }
return

RDP:
   StringTrimLeft, MNumber, A_GuiControl, 1
   MName := hosts[MNumber]
   Run, mstsc /v:%MName%
return

OpenLog:
   IfNotExist, %logFile%
      FileAppend,, %logFile%
   Run, %A_ScriptDir%\FailureLog.txt
return

ClearLog:
   WinClose, %logFile%
   IfExist, %logFile%
      FileDelete, %logFile%
return

FlushDNS:
   Run, flushdns.ahk
return

StartTrace:
   thost := A_GuiControl
   StringTrimLeft, thost, thost, 1
   StartTrace(hosts[thost], logFile)
return

GuiClose:
Exit:
   ExitApp
return

;<=====  Functions  ==========================================================>
CheckServer(host){
   objShell := ComObjCreate("WScript.Shell")
   objExec := objShell.Exec(ComSpec " /c ping -n 1 " host)
   strStdOut := ""
   while, !objExec.StdOut.AtEndOfStream
      strStdOut := objExec.StdOut.ReadAll()
   Loop, Parse, strStdOut, `n, `r
   {
      if (A_Index != 3)
         Continue
      else {
         RegExMatch(A_LoopField, "O)(?=|<)\d*ms", time)
         return time.Value()
      }
   }
}

StartTrace(host, file){
   Run, tracert.ahk %host%
}

LogE(host, file){
   FormatTime, TimeStamp, A_Now, [dd/MMM/yyyy HH:mm:ss]
   text := TimeStamp . " Failed to ping " . host . "`n"
   FileAppend, %text%, %file%
}
